package org.eclipse.equinox.launcher;

import java.io.IOException;
import java.net.JarURLConnection;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.jar.Manifest;

public class WebStartMain extends Main {
   private static final String PROP_WEBSTART_AUTOMATIC_INSTALLATION = "eclipse.webstart.automaticInstallation";
   private static final String DEFAULT_OSGI_BUNDLES = "org.eclipse.equinox.common@2:start, org.eclipse.core.runtime@start";
   private static final String PROP_OSGI_BUNDLES = "osgi.bundles";
   private static final String PROP_CHECK_CONFIG = "osgi.checkConfiguration";
   private Map allBundles = null;
   private List bundleList = null;

   public static void main(String[] args) {
      System.setSecurityManager((SecurityManager)null);
      int result = (new WebStartMain()).run(args);
      if (!Boolean.getBoolean("osgi.noShutdown")) {
         System.exit(result);
      }

   }

   private void setDefaultBundles() {
      if (System.getProperty("osgi.bundles") == null) {
         System.getProperties().put("osgi.bundles", "org.eclipse.equinox.common@2:start, org.eclipse.core.runtime@start");
      }
   }

   protected void basicRun(String[] args) throws Exception {
      this.setDefaultBundles();
      this.initializeBundleListStructure();
      this.discoverBundles();
      String fwkURL = this.searchFor(this.framework, (String)null);
      this.allBundles.remove(this.framework);
      System.getProperties().put("osgi.framework", fwkURL);
      super.basicRun(args);
   }

   protected void beforeFwkInvocation() {
      if (System.getProperty("osgi.checkConfiguration") == null) {
         System.getProperties().put("osgi.checkConfiguration", "true");
      }

      this.buildOSGiBundleList();
      this.cleanup();
   }

   private void cleanup() {
      this.allBundles = null;
      this.bundleList = null;
   }

   protected String searchFor(String target, String start) {
      ArrayList matches = (ArrayList)this.allBundles.get(target);
      if (matches == null) {
         return null;
      } else {
         int numberOfMatches = matches.size();
         if (numberOfMatches == 1) {
            return ((BundleInfo)matches.get(0)).location;
         } else if (numberOfMatches == 0) {
            return null;
         } else {
            String[] versions = new String[numberOfMatches];
            int highest = 0;

            for(int i = 0; i < versions.length; ++i) {
               versions[i] = ((BundleInfo)matches.get(i)).version;
               highest = this.findMax(versions);
            }

            return ((BundleInfo)matches.get(highest)).location;
         }
      }
   }

   private BundleInfo findBundle(String target, String version, boolean removeMatch) {
      ArrayList matches = (ArrayList)this.allBundles.get(target);
      int numberOfMatches = matches.size();
      if (numberOfMatches == 1) {
         return (BundleInfo)matches.remove(0);
      } else if (numberOfMatches == 0) {
         return null;
      } else if (version != null) {
         Iterator iterator = matches.iterator();

         while(iterator.hasNext()) {
            BundleInfo bi = (BundleInfo)iterator.next();
            if (bi.version.equals(version)) {
               iterator.remove();
               return bi;
            }
         }

         return null;
      } else {
         String[] versions = new String[numberOfMatches];
         int highest = 0;

         for(int i = 0; i < versions.length; ++i) {
            versions[i] = ((BundleInfo)matches.get(i)).version;
            highest = this.findMax(versions);
         }

         return (BundleInfo)matches.remove(highest);
      }
   }

   private void discoverBundles() {
      this.allBundles = new HashMap();

      try {
         Enumeration resources = WebStartMain.class.getClassLoader().getResources("META-INF/MANIFEST.MF");

         while(resources.hasMoreElements()) {
            BundleInfo found = this.getBundleInfo((URL)resources.nextElement());
            if (found != null) {
               ArrayList matching = (ArrayList)this.allBundles.get(found.bsn);
               if (matching == null) {
                  matching = new ArrayList(1);
                  this.allBundles.put(found.bsn, matching);
               }

               matching.add(found);
            }
         }
      } catch (IOException var4) {
         IOException e = var4;
         e.printStackTrace();
      }

   }

   private String extractInnerURL(URL url) {
      try {
         URLConnection connection = null;

         String var4;
         try {
            connection = url.openConnection();
            if (!(connection instanceof JarURLConnection)) {
               return url.toExternalForm();
            }

            var4 = "file:" + ((JarURLConnection)connection).getJarFile().getName();
         } finally {
            if (connection != null) {
               connection.getInputStream().close();
            }

         }

         return var4;
      } catch (IOException var8) {
         return url.toExternalForm();
      }
   }

   private void initializeBundleListStructure() {
      String prop = System.getProperty("osgi.bundles");
      if (prop != null && !prop.trim().equals("")) {
         this.bundleList = new ArrayList(10);
         StringTokenizer tokens = new StringTokenizer(prop, ",");

         while(tokens.hasMoreTokens()) {
            String token = tokens.nextToken().trim();
            String bundleId = token;
            if (!token.equals("")) {
               BundleInfo toAdd = new BundleInfo((BundleInfo)null);
               toAdd.bsn = bundleId;
               int startLevelSeparator;
               if ((startLevelSeparator = token.lastIndexOf(64)) != -1) {
                  toAdd.bsn = token.substring(0, startLevelSeparator);
                  toAdd.startData = token.substring(startLevelSeparator);
               }

               this.bundleList.add(toAdd);
            }
         }

      } else {
         this.bundleList = new ArrayList(0);
      }
   }

   private BundleInfo getBundleInfo(URL manifestURL) {
      try {
         Manifest mf = new Manifest(manifestURL.openStream());
         String symbolicNameString = mf.getMainAttributes().getValue("Bundle-SymbolicName");
         if (symbolicNameString == null) {
            return null;
         } else {
            BundleInfo result = new BundleInfo((BundleInfo)null);
            String version = mf.getMainAttributes().getValue("Bundle-Version");
            result.version = version != null ? version : "0.0.0";
            result.location = this.extractInnerURL(manifestURL);
            int pos = symbolicNameString.lastIndexOf(59);
            if (pos != -1) {
               result.bsn = symbolicNameString.substring(0, pos);
               return result;
            } else {
               result.bsn = symbolicNameString;
               return result;
            }
         }
      } catch (IOException var7) {
         IOException e = var7;
         if (this.debug) {
            e.printStackTrace();
         }

         return null;
      }
   }

   private void buildOSGiBundleList() {
      StringBuffer finalBundleList = new StringBuffer(this.allBundles.size() * 30);
      Iterator iterator = this.bundleList.iterator();

      while(iterator.hasNext()) {
         BundleInfo searched = (BundleInfo)iterator.next();
         BundleInfo found = this.findBundle(searched.bsn, searched.version, true);
         finalBundleList.append("reference:").append(found.location).append(searched.startData).append(',');
      }

      if (!Boolean.FALSE.toString().equalsIgnoreCase(System.getProperties().getProperty("eclipse.webstart.automaticInstallation"))) {
         iterator = this.allBundles.values().iterator();

         while(iterator.hasNext()) {
            ArrayList toAdd = (ArrayList)iterator.next();
            Iterator iterator2 = toAdd.iterator();

            while(iterator2.hasNext()) {
               BundleInfo bi = (BundleInfo)iterator2.next();
               finalBundleList.append("reference:").append(bi.location).append(',');
            }
         }
      }

      System.getProperties().put("osgi.bundles", finalBundleList.toString());
   }

   private class BundleInfo {
      String bsn;
      String version;
      String startData;
      String location;

      private BundleInfo() {
      }

      // $FF: synthetic method
      BundleInfo(BundleInfo var2) {
         this();
      }
   }
}
